from django.db import models

from canvas_sdk.v1.data.base import IdentifiableModel


class DetectedIssue(IdentifiableModel):
    """DetectedIssue."""

    class Meta:
        db_table = "canvas_sdk_data_api_detectedissue_001"

    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    identified = models.DateTimeField()
    deleted = models.BooleanField()
    originator = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.DO_NOTHING, null=True, related_name="+"
    )
    committer = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.DO_NOTHING, null=True, related_name="+"
    )
    entered_in_error = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.DO_NOTHING, null=True, related_name="+"
    )
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="detected_issues", null=True
    )
    code = models.CharField(max_length=20)
    status = models.CharField(max_length=16)
    severity = models.CharField(max_length=10)
    reference = models.CharField(max_length=200)
    issue_identifier = models.CharField(max_length=255)
    issue_identifier_system = models.CharField(max_length=255)
    detail = models.TextField()


class DetectedIssueEvidence(IdentifiableModel):
    """DetectedIssueEvidence."""

    class Meta:
        db_table = "canvas_sdk_data_api_detectedissueevidence_001"

    system = models.CharField(max_length=255)
    version = models.CharField(max_length=255)
    code = models.CharField(max_length=255)
    display = models.CharField(max_length=1000)
    user_selected = models.BooleanField()
    detected_issue = models.ForeignKey(
        DetectedIssue,
        on_delete=models.DO_NOTHING,
        related_name="evidence",
        null=True,
    )


__exports__ = ("DetectedIssue", "DetectedIssueEvidence")



from django.contrib.postgres.fields import ArrayField
from django.db import models

from canvas_sdk.v1.data.base import IdentifiableModel


class ReasonForVisitSettingCoding(IdentifiableModel):
    """ReasonForVisitSettingCoding."""

    class Meta:
        db_table = "canvas_sdk_data_api_reasonforvisitsettingcoding_001"

    objects: models.Manager["ReasonForVisitSettingCoding"]

    code = models.CharField(max_length=255)
    display = models.CharField(max_length=1000)
    system = models.CharField(max_length=255)
    version = models.CharField(max_length=255)

    duration = ArrayField(models.DurationField())


__exports__ = ("ReasonForVisitSettingCoding",)



import uuid
from collections.abc import Sequence
from decimal import Decimal


def quantize(deci: Decimal | float | str | tuple[int, Sequence[int], int]) -> Decimal:
    """Rounds a Decimal value to two decimal places."""
    return Decimal(deci).quantize(Decimal(".01"))


def create_key() -> str:
    """Generates a unique key using UUID4."""
    return uuid.uuid4().hex


__exports__ = ()



import json
from typing import Any

from pydantic_core import InitErrorDetails

from canvas_generated.messages.effects_pb2 import Effect
from canvas_sdk.base import TrackableFieldsModel
from canvas_sdk.v1.data import Patient


class PatientMetadata(TrackableFieldsModel):
    """Effect to upsert a Patient Metadata record."""

    class Meta:
        effect_type = "PATIENT_METADATA"

    patient_id: str
    key: str

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "patient_id",
                    f"Patient with id: {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        return errors

    def upsert(self, value: str) -> Effect:
        """Upsert the patient metadata."""
        self._validate_before_effect("upsert")

        return Effect(
            type=f"UPSERT_{self.Meta.effect_type}",
            payload=json.dumps(
                {
                    "data": {**self.values, "value": value},
                }
            ),
        )


__exports__ = ("PatientMetadata",)



from canvas_sdk.effects.patient.base import Patient, PatientContactPoint, PatientExternalIdentifier
from canvas_sdk.effects.patient.create_patient_external_identifier import (
    CreatePatientExternalIdentifier,
)

__all__ = __exports__ = (
    "Patient",
    "PatientContactPoint",
    "PatientExternalIdentifier",
    "CreatePatientExternalIdentifier",
)
